#!/usr/bin/env perl

#
# $Id$
#

# Copyright INRA

# Mathieu.Gautier@supagro.inra.fr
# Astrid.Cruaud@supagro.inra.fr
# Jean-Yves.Rasplus@supagro.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#

=head1 NAME

RADIS_step2_data_analyses.pl

=head1 SYNOPSIS

$RADIS/bin/RADIS_step2_data_analyses.pl [--cfg filename] --outdir rooted_dirname [--workingdir rooted_dirname] [--verbose] [--ncpus integer]

=head1 DESCRIPTION

=cut

BEGIN
{
	$ENV{LANG}="C";
	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../lib");
	$ENV{RADIS} = "$dirprg/.." if ( ! defined ($ENV{RADIS}) );
	our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

use Getopt::Long;
use ParamParser;
use Data::Dumper;
use File::Path;
use File::Copy;
use General;
use GeneralBioinfo;
require "radis-lib.pl";
require "radis-libR.pl";

use strict;
use warnings;

my $DEFAULT_NCPUS = 2;
my $LOGGER_LEVEL_EXEC = 'DE';
my $DEBUG=0;

MAIN:
{
	my $o_param =  New ParamParser('GETOPTLONG',\&Usage,"help","cfg=s","outdir=s",'workingdir=s',"verbose","ncpus=i",'listoffastqfiles=s','radis_npbloci_cutoff=i','radis_bwa_seedlen=i','building_loci','loci_analysis','debug','without-raxml');
	$o_param->SetBehaviour('use_substitution_table');
	$o_param->SetUnlessDefined('cfg',"$ENV{RADIS}/cfg/RADIS.cfg");
	$o_param->AssertFileExists('cfg','outdir');
	$o_param->AssertDirExists('outdir');
	$o_param->AssertFullPath('outdir','cfg');
	if ( $o_param->IsDefined('workingdir') )
	{
		$o_param->AssertDirExists('workingdir');
		$o_param->AssertFullPath('workingdir');
	}
	#$o_param->AssertInteger('radis_bwa_seedlen');
	$o_param->SetSubstitution('%i',\$ENV{RADIS});
	# pourquoi pas $o_param->SetSubstitution('%i',$ENV{RADIS});
	$o_param->SetUnlessDefined('ncpus',$DEFAULT_NCPUS);
	$o_param->SetSubstitution('%n',$o_param->Get('ncpus'));
	$o_param->Update($o_param->Get('cfg'),'U');
	my $mlist =  $o_param->Get('stacks_ustacks_M');
	&NoSpace(\$mlist);
	my $nlist =  $o_param->Get('stacks_cstacks_n');
	&NoSpace(\$nlist);
	my @a_m= split(',',$mlist);
	my @a_n = split(',',$nlist);
	&AssertInteger($_) foreach @a_m;
	&AssertInteger($_) foreach @a_n;

	my $nloci_min=$o_param->Get('radis_nloci_min');
	&AssertInteger($nloci_min);

	my $nsample_min_list =  $o_param->Get('radis_nsample_min');
	&NoSpace(\$nsample_min_list);
	my @a_nsample_min= split(',',$nsample_min_list);
	&AssertInteger($_) foreach @a_nsample_min;

	my $radis_npbloci_cutoff=$o_param->Get('radis_npbloci_cutoff');
	&AssertInteger($radis_npbloci_cutoff);


	my $outdir  = $o_param->Get('outdir');
	$o_param->SetUnlessDefined('workingdir',$outdir);
	my $workdir = $o_param->Get('workingdir');
	my $ncpus=$o_param->Get('ncpus');
	$o_param->AssertDefined('listoffastqfiles','radis_npbloci_cutoff');

	print "Main parameters\n";
	print "radis_nloci_min=$nloci_min\n";
	print "radis_nsample_min=$nsample_min_list\n";
	print "radis_npbloci_cutoff=$radis_npbloci_cutoff\n\n";

	$main::DEBUG = 1 if ( $o_param->IsDefined('debug') );

	my $running_mode = '';
	$running_mode .= ' building_loci ' if ( $o_param->IsDefined('building_loci') );
	$running_mode .= ' loci_analysis ' if ( $o_param->IsDefined('loci_analysis') );
	if ($running_mode eq '' ) # si aucune etape n'est specifie, alors je fais les deux
	{
		$running_mode = 'building_loci loci_analysis';
	}
#	chdir($workdir);
#	$o_param->Print();

	#====== ustacks ==============

	my $ra_lof = $o_param->GetListOfFiles('listoffastqfiles');
	if ( !scalar(@$ra_lof) )
	{	
		print STDERR "Error: no file found\n";
		exit(1);
	}
	elsif ( scalar(@$ra_lof) == 1 )
	{
		print STDERR "Error: only one sample file found, please check the validity of --listoffastqfiles (maybe the quotes are missing)\n";
		exit(1);
	}
	my %h_sample2id =(); 
	my %h_id2sample =(); 
	# je vais faire une table de correspondance sample / ID (valeur numerique)
	# ca me permettra de ne pas avoir les soucis de consistence evoques dans le doc
	my $cptid=1;
	my %h_ids=();
	for(my $i=0;$i<scalar(@$ra_lof);$i++)
	{
		my $nm = &GetSampleName($$ra_lof[$i]);
		$h_ids{$nm}=1;
	}
	foreach my $nm(sort keys(%h_ids)) # je fais en 2 coup pour bien trier les samples
	{
		if ( ! defined($h_sample2id{$nm}) )
		{
			$h_sample2id{$nm}=$cptid;
			$h_id2sample{$cptid}=$nm;
			$cptid++;
		}
	}

	my $run_stacks = ( $running_mode =~ /building_loci/ ) ? &TRUE: &FALSE;
	my $run_phylo = ( $running_mode =~ /loci_analysis/ ) ? &TRUE: &FALSE;

	my $msg = sprintf("Step 2.1: ustacks (%d files)",scalar(@$ra_lof));
	&PrintJobStatus($msg,"started",&TRUE) if ( $run_stacks );
	my @a_cde = ();
	#
	# Loop #1: ustacks_M
	#
	foreach my $ustacksM (@a_m)
	{
		if ( scalar(@a_m) > 1 && $run_stacks )
		{
			&PrintTitle("================= ustacks M: $ustacksM  =====================");
		}
		my $loutdir = "$outdir/M$ustacksM";
		mkdir $loutdir if ( ! -d $loutdir );

		my $outdir1="$loutdir/01_TAGS";
		if ($run_stacks)
		{
			mkdir $outdir1 if ( ! -d $outdir1 );
			for(my $i=0;$i<scalar(@$ra_lof);$i++)
			{
				my $nm = &GetSampleName($$ra_lof[$i]);
				push(@a_cde,$o_param->Get('stacks_ustacks')." -M $ustacksM -f $$ra_lof[$i] -o $outdir1 -i $h_sample2id{$nm}");
			}
			&Multithreading(\&SliceSystem,$ncpus,\@a_cde,undef);
			# a ce stade les fichiers contiennent les codes des ID en colonne 2
			$msg = sprintf("Step 2.1: ustacks");
			&PrintJobStatus($msg,"done",&TRUE);
		}

		my $outdir2 = "$loutdir/02_CATALOG";
		mkdir $outdir2;
		#
		# Loop #2: cstacks_n
		#
		foreach my $n (@a_n)
		{
			if ( scalar(@a_n) > 1 )
			{
				&PrintTitle("============= ustacks M: $ustacksM / cstacks n: $n  ===============");
			}
			if ( $run_stacks )
			{
				$msg = sprintf("Step 2.2: building catalog");
				&PrintJobStatus($msg,"started",&TRUE);
				my $cde;
				for(my $i=0;$i<scalar(@$ra_lof);$i++)
				{
					my $nm = &GetSampleName($$ra_lof[$i]);
					$cde = "cd $outdir1 ; ".$o_param->Get('stacks_cstacks')." -n $n -b $n -o $outdir2 -s $nm"; 
					if ( $i != 0 )
					{
						$cde .= " --catalog $outdir2/batch_$n.catalog ";
					}
					&System($cde,${main::DEBUG},undef);
				}
				&PrintJobStatus($msg,"done",&TRUE);
			}

			# je fais l'etape a la fois dans run_stacks & loci_analysis car les fichiers sont generes/utilises => VERIF
			$msg = sprintf("Step 2.3: selection");
			&PrintJobStatus($msg,"started",&TRUE);
			my %h_ind2count=();
			&BuildMatrix("$outdir2/batch_$n.catalog.tags.tsv","$outdir2/batch_$n.catalog.matrix.tsv",\%h_id2sample,\%h_ind2count);
#TAG JRAS03787 JRAS03886 JRAS03952 JRAS03954
# 0 0 0 0
#1 0 1 0 0
			&CleanupMatrix("$outdir2/batch_$n.catalog.matrix.tsv"); # lors du passage v1.32 creation d'une ligne avec un tag vide, je supprime les lignes ou il n'y a pas le meme nb de col que pour le header
			&CatalogStatR($outdir2,"$outdir2/batch_$n.catalog.matrix.tsv");
			&CheckOutputFiles("Step 2.3:","$outdir2/batch_$n.catalog.matrix.tsv.summary");

			my $outdir3 = "$loutdir/n${n}_03_SEL";
			mkdir $outdir3;

			foreach my $nsample_min (@a_nsample_min)
			{
				if ( scalar(@a_nsample_min) > 1 )
				{
					&PrintTitle("============= ustacks M: $ustacksM / cstacks n: $n / nsample_min : $nsample_min  ===============");
				}
				my $selection_stacks = "$outdir3/stacks_M${ustacksM}n${n}S${nsample_min}L$nloci_min.sel";
				&FilterMatrixR("$outdir2/batch_$n.catalog.matrix.tsv",$outdir3,$selection_stacks,\%h_ind2count,$nloci_min,$nsample_min);
				$msg = sprintf("Step 2.3: selection $selection_stacks ");
				&PrintJobStatus($msg,"done",&TRUE);

				next if ( ! $run_phylo );

				my @a_lstdir = ();
				push(@a_lstdir,$loutdir);

				my $cutoff_paralog=$o_param->Get('radis_npbloci_cutoff');

				if ( $cutoff_paralog != 0 )
				{
					my $noparalog_dir = "$loutdir/NO_PBLOCI-$cutoff_paralog";
					mkdir $noparalog_dir;

					my $with_tags_for_paralogs=undef;
					if ( defined($with_tags_for_paralogs = &TestParalogieR($noparalog_dir,"$outdir2/batch_$n.catalog.matrix.tsv",$cutoff_paralog)) )
					{
						my $selection_all_stacks=$selection_stacks;
						push(@a_lstdir,$noparalog_dir);
						my $outdir3 = "$noparalog_dir/n${n}_03_SEL";
						mkdir $outdir3;
						# je n'utilise pas les scripts awk, je peux faire plus simple pour me recaler
						my %h_stackswithparalops=();
	#,JRAS03802,JRAS03806,JRAS03807,JRAS03866,JRAS03877,JRAS03878,JRAS03881,JRAS03884,JRAS03887,JRAS03891,JRAS03906,JRAS03917,JRAS03921,JRAS03935,JRAS03940,JRAS03947,JRAS03951,JRAS03956,JRAS03958,JRAS03959,JRAS03963,JRAS05534,JRAS05540,JRAS05565,JRAS05569,JRAS05571,JRAS05574,JRAS05582,JRAS05586,JRAS05588,JRAS05669,effectif
	#1,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,0
	#3,NA,NA,1,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,1
						my $f_paralogs = &GetStreamIn($with_tags_for_paralogs);
						my $l=<$f_paralogs>;#header
						while(my $l=<$f_paralogs>)
						{	
							chomp($l);
							my @a_line = split(',',$l);
							if ($a_line[$#a_line] > 0 )
							{
								$h_stackswithparalops{$a_line[0]}=1;
							}
						}
						$f_paralogs->close;
						my $f_full      = &GetStreamIn($selection_all_stacks);
						$selection_stacks = "$outdir3/stacks_M${ustacksM}n${n}S${nsample_min}L$nloci_min.sel";  # meme nom mais dans une arborescence NO-PARALOG
						my $f_noparalog = &GetStreamOut($selection_stacks); # meme nom mais dans une arborescence NO-PARALOG-cutoff_paralog
						while(my $l=<$f_full>)
						{
							chomp($l);
							print $f_noparalog "$l\n" if ( ! defined($h_stackswithparalops{$l}));
						}
						$f_full->close;
						$f_noparalog->close;
					}
				}
				#
				# Loop #2: with and without paralogs (if any)
				#
#				print STDERR Dumper(@a_lstdir);
				my $c=0;
				foreach my $outdir (@a_lstdir)
				{
					my $f_err = ( $c == 0 ) ? *STDERR : undef;
					my $msg = sprintf("%s",( $c == 0 ) ? "With problematic loci": "Without problematic loci");
					$c++;
					print "============= $msg =================\n";
					my $outdir3 = "$outdir/n${n}_03_SEL";
					$selection_stacks = "$outdir3/stacks_M${ustacksM}n${n}S${nsample_min}L$nloci_min.sel";
					$msg = sprintf("Step 2.4: building phylip files ");
					&PrintJobStatus($msg,"started",&TRUE);
					my $phylip_file_basename="stacks_M${ustacksM}n${n}S${nsample_min}L$nloci_min.sel.phy";
					my $phylip_file = "$outdir3/$phylip_file_basename";
					my $outdir4 = "$outdir/n${n}_04_LOCI";
					#mkdir $outdir4;
					#mkdir "$outdir4/BYSAMPLE";
					#mkdir "$outdir4/BYLOCI";
					my $outdir5 = "$outdir/n${n}_05_GENREF";
					#mkdir $outdir5;
					&BuildingAlignmentAndLocusbasedFiles("$outdir2/batch_$n.catalog.tags.tsv",$selection_stacks,$outdir1,\%h_id2sample,\%h_ind2count,$nloci_min,$phylip_file,$outdir4,$outdir5,$f_err);
					&CheckOutputFiles("Step 2.4:","$phylip_file");
					#&CheckOutputFiles("Step 2.4:","$outdir4/BYSAMPLE/*.fa","done",&TRUE);
					#&CheckOutputFiles("Step 2.4:","$outdir4/BYLOCI/*.phy");
					#&CheckOutputFiles("Step 2.4:","$outdir5/*_ref.fa");
	#				&CheckOutputFiles("Step 2.4:","$outdir5/longest_ref.fa"); # mettre stacks_M${ustacksM}n${n}i${nsample_min}t$nloci_min.sel ?

					if ( ! $o_param->IsDefined('without-raxml') )
					{
						$msg = sprintf("Step 2.5: RAxML ");
						&PrintJobStatus($msg,"started",&TRUE);
						my $partition = $o_param->Get('raxml_partition');
						my $raxml_file = "$phylip_file.raxml";
						my $cde = "cd $outdir3 ; ".$o_param->Get('raxml_prg')." ".$o_param->Get('raxml_params')." $partition -s $phylip_file_basename -n $phylip_file_basename.raxml > $phylip_file_basename.raxml.stdout 2> $phylip_file_basename.raxml.stderr";
						&System($cde,${main::DEBUG},undef);
						&CheckOutputFiles("Step 2.5:","$outdir3/*$phylip_file_basename.raxml");
					}

					#$msg = sprintf("Step 2.6: computation of distance distribution ");
					#&PrintJobStatus($msg,"started",&TRUE);
					#&DistanceR("$outdir4/BYLOCI",".locus.phy");
					#&CheckOutputFiles("Step 2.6:","$outdir4/BYLOCI/RESULTS.Distance.*.pdf");
				}
			}
		}
	}
	exit(0);
}
			

sub CleanupMatrix
{
my($mat)=@_;

	&System("mv $mat $mat.raw",${main::DEBUG},undef);
	my $f_in  = &GetStreamIn("$mat.raw");
	my $header = <$f_in>;
	my $nb = scalar(split(' ',$header));
	my $f_out = &GetStreamOut("$mat");
	print $f_out $header;
	while(my $l=<$f_in>)
	{
		next if ( $nb != scalar(split(' ',$l)) );
		print $f_out $l;
	}
	$f_in->close;
	$f_out->close;
}

sub CheckOutputFiles
{
my($step,$lscmd)=@_;

	my @a_ls = `ls $lscmd 2> /dev/null`;
	my $nbfiles = scalar(@a_ls);
	my $txtfile = ( scalar(@a_ls) <= 1 ) ? 'file' : 'files';
	&PrintJobStatus("$step $lscmd ($nbfiles $txtfile) ","done",&TRUE);
	if ( scalar(@a_ls) < 1 )
	{
		&PrintJobStatus(" Warning : no file found ","",&TRUE);
	}
}

sub PrintTitle
{
my($msg)=@_;

	my $len = length($msg);
	my $str = '=' x $len;
	print "$str\n$msg\n$str\n";
}

#sub Mapping
#{
#my($o_param,$outdir,$ref,$lof_targets,$seedlen)=@_;
#
#	&System($o_param->Get('bwa_index')." $ref ",&FALSE,undef);
#	&System($o_param->Get('samtools')." faidx $ref ",&FALSE,undef);
#	my($basenameref)=$ref =~ /.+\/(.+)/;
#	my @a_targets=`ls $lof_targets`;
#	chomp(@a_targets);
#	print STDERR "Warning: don't worry about warning: >[bam_header_read] EOF marker is absent. The input is probably truncated<\n";
#	foreach my $target (@a_targets)
#	{
#		my ($dir,$file) = $target =~ /(.+)\/(.+)/;
#		&System($o_param->Get('bwa_mem')." -k $seedlen $ref $target > $outdir/$basenameref.vs.$file.sam",&FALSE,undef);
#		# conversion et tri sans creation des fichiers intermediaires
#		&System("cd $outdir ; ".$o_param->Get('samtools')." view -bS $outdir/$basenameref.vs.$file.sam | ".$o_param->Get('samtools')." sort - $basenameref.vs.$file.sorted ",&FALSE,undef);
#		&System("cd $outdir ; ".$o_param->Get('samtools')." index $basenameref.vs.$file.sorted.bam ",&FALSE,undef);
#	}
#}

# 5.4.2, 5.5.1 5.5.2, 5.5.3
# extraction de l'alignement de tous les loci
# extraction des tags par sample au format fasta
# extraction des alignements loci par loci

# 20150622: iteration3 je supprime ce qui est dans outdir_loci & outdir_genref
sub BuildingAlignmentAndLocusbasedFiles
{
my($catalog,$selected_stacks,$outdir1,$rh_id2sample,$rh_ind2count,$nloci_min,$phylip_file,$outdir_loci,$outdir_genref,$f_err)=@_;

#	print "($catalog,$selected_stacks,$outdir1,$rh_id2sample,$phylip_file,$outdir_loci,$outdir_genref,$f_err)\n";

	# je charge les stacks ok
	my %h_stack=();
	my $f_sel = &GetStreamIn($selected_stacks);
	while(my $l=<$f_sel>)
	{
		chomp($l);
		$h_stack{$l}=1;
	}
	$f_sel->close;
	my $f_cat = &GetStreamIn($catalog);
	my %h_sampletags=(); # {SAMPLENAME}{TAG}=STACKID
	my %h_stacklen=();   # {STACKID}=LEN
	while(my $l=<$f_cat>)
	{
		next if ( $l =~ /^#/ );
		chomp($l);
		my %h_count=();
		my @a_=split(/\t/,$l);
		my $stackid = $a_[2];
		next if ( ! defined($h_stack{$stackid}) ); # je passe la stack n'est pas selectionnee
		my ($stack,$consensus) = $l =~ /consensus\s+\S+\s+(\S+)\s+(\S+)/;
		foreach my $tag (split(',',$stack))
		{
			my($sampleid,$tagid)=split('_',$tag);
			my $samplenm = $$rh_id2sample{$sampleid};
			if ( ! &Exists(\%h_sampletags,$samplenm,$tagid) ) # pour ne considerer que le 1er tag dans le cas ou il y en a plusieurs par sample
			{
				$h_sampletags{$samplenm}{$tagid}=$stackid;
			}
			$h_stacklen{$stackid}=length($consensus);
		}
	}
	$f_cat->close;
	my %h_samplestacks=(); # {SAMPLENAME}{STACKID}=SEQ
	my @a_ls = `ls $outdir1/*.tags.tsv`;
	chomp(@a_ls);
	foreach my $sampletags (@a_ls)
	{
		my $samplenm = &GetSampleName($sampletags);
#       1       2               0       +       consensus                       CCGCTACTGCCAGCCTCTTCTCTGACTCGATCGTCTCTAAGGTAAGAGAACAATGCTGCCTCTACAATATGTAGCCGTCATCGTCTGTG       0       0       0
		my $f_in = &GetStreamIn($sampletags);
		while(my $l=<$f_in>)
		{
			chomp($l);
			next if ( $l !~ /consensus\s+(\w+)/ );
			my $seq = $1;
			my @a_ =split(/\t/,$l);
			next if ( ! &Exists(\%h_sampletags,$samplenm,$a_[2]) );
			my $stackid = $h_sampletags{$samplenm}{$a_[2]};
			next if ( ! defined($h_stack{$stackid}) );
			if ( length($seq) != $h_stacklen{$stackid} )
			{
				if ( defined($f_err) )
				{
					print $f_err "Warning: Inconsistent length for $samplenm in stack=$stackid  length_in_sample=".length($seq)." != stack_consensus_length=$h_stacklen{$stackid}\t$l\n";
				}
			}
			else
			{
				$h_samplestacks{$samplenm}{$stackid}=$seq;
			}
		}
		$f_in->close;
	}

	my $nb_samples = 0;
	foreach  my $samplenm (keys(%h_samplestacks))
	{
		next if ( $$rh_ind2count{$samplenm} < $nloci_min );
		$nb_samples++;
	}
	my $nb_nt = 0;
	my $f_phy_raw=&GetStreamOut("$phylip_file.raw");
	my $f_phy=&GetStreamOut("$phylip_file");
	my $header = undef;
	my $missing_data=0;

	my %h_samplelen=();
	# alignement de tous les loci au format phylip 
	# extraction des loci par sample au format fasta
	foreach my $samplenm (sort keys(%h_samplestacks))
	{
		next if ( $$rh_ind2count{$samplenm} < $nloci_min );

#		my $f_fa = &GetStreamOut("$outdir_loci/BYSAMPLE/$samplenm.loci.fa");
#		my $f_reffa = &GetStreamOut("$outdir_genref/${samplenm}_ref.fa");
		my $seq = '';
		print $f_phy_raw "$samplenm\t";
#		print $f_reffa ">$samplenm\n";
		foreach my $stackid (sort keys(%h_stack))
		{
			if ( &Exists(\%h_samplestacks,$samplenm,$stackid) )
			{
				print $f_phy_raw $h_samplestacks{$samplenm}{$stackid};
#				print $f_fa ">$samplenm.$stackid\n$h_samplestacks{$samplenm}{$stackid}\n";
#				print $f_reffa "$h_samplestacks{$samplenm}{$stackid}";
				$h_samplelen{$samplenm}+=length($h_samplestacks{$samplenm}{$stackid});
			}
			else
			{
				print $f_phy_raw '?' x $h_stacklen{$stackid};
#				print $f_reffa 'N' x $h_stacklen{$stackid};
				$missing_data += $h_stacklen{$stackid};
			}
			if( ! defined($header) )
			{
				$nb_nt += $h_stacklen{$stackid};
			}
		}
		print $f_phy_raw "\n";
		if ( ! defined($header) )
		{
			$header = "$nb_samples\t$nb_nt";
			print $f_phy "$header\n";
		}
#		$f_fa->close;
#		print $f_reffa "\n";
#		$f_reffa->close;
	}
	$f_phy_raw->close;
	$f_phy->close;

	my @a_sample_len_sorted = sort { $h_samplelen{$b} <=> $h_samplelen{$a}  } keys(%h_samplelen);
	my $reference_sample = $a_sample_len_sorted[0];

#	&System("cd $outdir_genref ; rm -f longest_ref.fa ; ln -s ${reference_sample}_ref.fa longest_ref.fa",${main::DEBUG},undef);

	if ( $nb_nt == 0 )
	{
		printf "\nno data, check radis_nloci_min, radis_nsample_min parameters, maybe there are too stringent for this dataset\n\n";
		exit(1);
	}

	printf "%% of missing data= %.3f ($missing_data*100)/($nb_samples*$nb_nt)\n",($missing_data*100)/($nb_samples*$nb_nt);

	# extraction des stacks au format phylip
	foreach my $stackid (sort keys(%h_stacklen))
	{
		my $nb_sample=0;
		# 1er passage je compte le nombre de sample sans donnees manquantes
		# je ne sais pas si c'est trop couteux ou pas, on verra
		foreach my $samplenm (sort keys(%h_samplestacks))
		{
			next if ( $$rh_ind2count{$samplenm} < $nloci_min );

			$nb_sample++ if ( &Exists(\%h_samplestacks,$samplenm,$stackid) );
		}
		if ( 0 )
		{
			my $f_phy = &GetStreamOut("$outdir_loci/BYLOCI/$stackid.locus.phy");
			print $f_phy "$nb_sample\t$h_stacklen{$stackid}\n";
			foreach my $samplenm (sort keys(%h_samplestacks))
			{
				next if ( $$rh_ind2count{$samplenm} < $nloci_min );

				if ( &Exists(\%h_samplestacks,$samplenm,$stackid) )
				{
					print $f_phy "$samplenm.$stackid\t$h_samplestacks{$samplenm}{$stackid}\n";
				}
			}
			$f_phy->close;
		}
	}

	# extraction des alignements par loci au format phylip
	&System("cat $phylip_file.raw >> $phylip_file",${main::DEBUG},undef);
	&System("rm $phylip_file.raw");
}


sub BuildMatrix
{
my($in,$out,$rh_id2sample,$rh_ind2count)=@_;

	my $f_out = &GetStreamOut($out);
	print $f_out "TAG";
	foreach my $c (sort (values(%$rh_id2sample)))
	{
		print $f_out " $c";
	}
	print $f_out "\n";
#attention la colonne 4 est vide
#pour eviter, je vais m'ancrer sur consensus
#0       2       1               0       +       consensus       0       1_1,22_263,28_604       TTAGCATACCTTTTAGAGTACTCATTACGCGTTCTACCTGCCCGTTGGCGCGGGGACCTCCGGTAGCGATTAGGTGCAAAGCAATATTG       0       0       0
#0       2       2               0       +       consensus       0       1_2     ATGGGATTATGATTCACTACCTGGCATTTAGCACCAATTCCGCATGTTCCGGGGCACGGGTTTATACATTTTTGATTTACACAAGCCTC       0       0       0
	my $f_in  = &GetStreamIn($in);
	while(my $l=<$f_in>)
	{
		next if ( $l =~ /^#/ );
		chomp($l);
		my %h_count=();
		my ($stack) = $l =~ /consensus\s+\S+\s+(\S+)/;
		my @a_=split(/\t/,$l);
		foreach my $tag (split(',',$stack))
		{
			my($sampleid,$tagid)=split('_',$tag);
			$h_count{$$rh_id2sample{$sampleid}}++;
#			print STDERR "$tag, >$stack< $sampleid,$tagid : $$rh_id2sample{$sampleid}\n";
		}
		print $f_out "$a_[2]";
		foreach my $c (sort (values(%$rh_id2sample)))
		{
			my $count = $h_count{$c} || 0;
			$$rh_ind2count{$c}++ if ( $count );
			print $f_out " $count";
		}
		print $f_out "\n";
	}
	$f_in->close;
	$f_out->close;
}

sub GetSampleName
{
my($path)=@_;

	my($d,$f)=$path=~/(.+)\/(.+)/;
	$f=~s/\..+//;
	return $f;
}

=head2 procedure Usage
 
 Title        : Usage
 Usage        : &Usage()
 Prerequisite : none
 Fonction     : print the program usage
 Returns      : none
 Args         :	none
 Globals      : none

=cut

sub Usage
{
my $ncpus = $DEFAULT_NCPUS;

print STDERR<<END
$0
	--help
[Mandatory]
	--listoffastqfiles 'quoted string'
	--outdir     dirname 		
[Optional]
	--cfg        filename  		configuration file
	--ncpus      integer        [$ncpus] number of cpus
	--building_loci             stop after the generation of loci (cstacks/ustacks based)
	--without-raxml             skip RAxML analyses
	--radis_npbloci_cutoff interger
END
;
#--loci_analysis             runs the phylogenetic part of the pipeline  Warning: requires files generated by building_loci
}
